#!/usr/bin/perl -w
############################## check_snmp_hp5400_temp.pl ########
my $Version='1.0';
# Date : 19 février 2019
# Author  : Nicolas Biersart
# Help : support@esia-sa.com
#################################################################

use strict;
use Net::SNMP;
use Getopt::Long;
use lib "/usr/local/esia/plugins/lib/perl"  ;
use ESIA::SNMP;
use ESIA::PLUGIN;
use ESIA::INVENTORY;
use Data::Dumper;
use JSON;
use Encode;
use utf8;
binmode(STDOUT, ":utf8");
my $PROGNAME='get_snmp_inventory_default.pl';
my $PROGDESC = "get par défaut du module d'inventaire";
my $TIMEOUT = $ESIA::PLUGIN::TIMEOUT;
my %ERRORS= %ESIA::PLUGIN::ERRORS;
my $json;
my $return=$ERRORS{OK};
my ($opt_h,$opt_V);
my $os_type = "Debian";
my $oid_soft_installed = "1.3.6.1.2.1.25.6.3.1.2";
my $oid_soft_date = "1.3.6.1.2.1.25.6.3.1.5";
my $oid_soft_type = "1.3.6.1.2.1.25.6.3.1.4";
my $oid_sysdescr = "1.3.6.1.2.1.1.1.0";
my $oid_windows_version_major = "1.3.6.1.4.1.77.1.1.1.0";
my $oid_windows_version_minor = "1.3.6.1.4.1.77.1.1.2.0";
#
#MAIN
#
&Getopt::Long::config('bundling');
#on récupère les paramètres SNMP
my ($snmp, $snmperror) = ESIA::SNMP::SessionAutoCommunityVersion(@ARGV);

GetOptions(
	"V"   => \$opt_V,	"version"    => \$opt_V,
	"h"   => \$opt_h,	"help"       => \$opt_h,
	"J" => \$json,	 "json" => \$json,
	"T=s" => \$os_type,	 "type=s" => \$os_type,
);
# -h & --help print help
if ($opt_h)
{
    #affiche l'aide par défaut
    ESIA::SNMP::print_snmp_usage($PROGNAME,'-T Debian');
    my @arr;
	push(@arr,["-T","--type","type of test [cpu,mem,env] (default: cpu)" ]);
	push(@arr,["-w","--warning","warning threshold in percent" ]);
	push(@arr,["-c","--critical","critical threshold in percent"]);
	ESIA::SNMP::print_snmp_help($PROGNAME,$PROGDESC,\@arr);
	exit $ERRORS{'OK'};
}
sub spaced_hex {
    join(' ', unpack '(A2)*', sprintf('%08X', shift));
}
sub convert_hexa{
	my ($string) = @_;
	if($string =~ /0x([a-fA-F0-9][a-fA-F0-9]*)/){

		my $to_convert = $1;
		## convert each two digit into char code
		$to_convert =~ s/([a-fA-F0-9][a-fA-F0-9])/chr(hex($1))/eg;

		$string =~ s/0x([a-fA-F0-9][a-fA-F0-9]*)/$to_convert/g;
	}
	return $string;
}

#on vérifie qu'on a bien un objet SNMP
if (!defined($snmp))
{
	print ("UNKNOWN: SNMP configuration error: $snmperror\n");
	exit $ERRORS{'UNKNOWN'};
}
#$snmp->translate([-octetstring => 0]);
my @output;
my $OS_name="$os_type";
my $OS_version="";
my $OS_editor="";
my $result = $snmp->get_table( $oid_soft_installed);
my $sysdresc = ESIA::SNMP::get($snmp,"$oid_sysdescr");
if($sysdresc)
{
	foreach my $key ( sort keys %$result)
	{
		    #on récupère l'index
		if($key=~ /([0-9]*)$/ )
		{
			my $index =$1;
			my $fullname =$$result{"$key"};
		    if($fullname=~/0[xX]/)
		    {
		        #$fullname=hex_to_ascii($fullname);
		        $fullname = convert_hexa($fullname);
		        $fullname = decode("iso8859-1", $fullname);

		    }
			my $octet = ESIA::SNMP::get($snmp,"$oid_soft_date.$index");
		    my $datetime = 'unknown';
		    if($octet ne '0x0000010100000000')
		    {
		        my @date = unpack 'n C6 a C2', $octet;
		        $datetime = sprintf "%04d-%02d-%02d %02d:%02d:%02d.%d%s%02d:%02d", @date;
		    }
		    my $soft_type = ESIA::SNMP::get($snmp,"$oid_soft_type.$index");
		    my $soft_type_name='SOFTWARE';
		    my $version='';
		    my $name = $fullname;
		    my $editor="";
		    if($os_type eq 'Debian')
		    {
		        #récupération de la version Debian
		        if($fullname =~/\-([0-9\.]*)\-/)
		        {
		            $version= $1;
		        }
		        elsif($fullname =~/\-([0-9\.]*)$/)
		        {
		            $version= $1;
		        }
		        elsif($fullname =~/\-([0-9\.]*)\+/)
		        {
		            $version= $1;
		        }
		        # récupération du nom du paquet
		        if($fullname =~/^([a-zA-Z\.\-]*)/)
		        {
		            $name =$1;
		            if($name=~/^(.*)\-$/)
		            {
		                $name =$1;
		            }
		            elsif($fullname=~/^([a-zA-Z\.\-]*)([0-9\.]*)([a-zA-Z\.\-]*)\-/)
		            {
		                 $name =$1.$2.$3;
		            }
		        }
		        if($soft_type == 2)
		        {
		            $soft_type_name = 'OS-COMPONENT';
		            if($fullname =~/^base\-files\-[0-9\.]*\+deb([0-9]*)u([0-9]*)/)
		            {
		                $OS_version="$1.$2";
		               	$OS_editor="Debian";
		            }
		        }
		        else
		        {
		            if($name =~/^lib.*/)
		            {
		                $soft_type_name='LIBRARY';
		            }
		            elsif($name =~/\-common/)
		            {
		                $soft_type_name='LIBRARY';
		            }
		            elsif($name =~/\-[data]/)
		            {
		                $soft_type_name='APPLICATION-DATA';
		            }
		            elsif($name =~/\-[module|contrib]/)
		            {
		                $soft_type_name='APPLICATION-MODULES';
		            }
		            elsif($name =~/^php[0-9]\.[0-9]\-/)
		            {
		                $soft_type_name='LIBRARY';
		                $editor="The PHP Group";
		            }
		            elsif($name =~/^php\-[a-zA-Z]*\-/)
		            {
		                $soft_type_name='LIBRARY';
		                $editor="The PHP Group";
		            }
		            elsif($name =~/^python[0-9]\-[a-zA-Z]*/)
		            {
		                $soft_type_name='LIBRARY';
		                $editor="Python Software Foundation";
		            }
		            elsif($name =~/^ruby[0-9]\-[a-zA-Z]*/)
		            {
		                $soft_type_name='LIBRARY';
		                $editor="Ruby";
		            }

		        }
		    }
		    elsif($os_type eq 'Microsoft Windows')
		    {
		    	if($OS_editor eq '')
		    	{
		    		$OS_editor="Microsoft Corporation";
		    		my $ver = ESIA::SNMP::get($snmp,"$oid_windows_version_major");
		    		my $vermin = ESIA::SNMP::get($snmp,"$oid_windows_version_minor");
		    		$OS_version = "$ver.$vermin";
		    	}
		    }
		    #print "$name :\t\t$soft_type_name \t$datetime\n";
		    my %hash = ('name' => $name,'manufacturer'=>$editor, 'type'=>$soft_type_name,'version'=>$version,'dateInstalled' => $datetime,'fullname'=>$fullname);
		    push(@output, \%hash);
		}
	}
	my %hash = ('name' => $OS_name,'fullname'=>$sysdresc,'manufacturer'=>$OS_editor, 'type'=>'OS','version'=>$OS_version);
	push(@output, \%hash);
}
if($json)
{
    my $out = encode_json(\@output);
    if($out eq "")
    {
        $return = $ERRORS{CRITICAL};
    }
    else
    {
        print "$out";
    }
}
else
{
            foreach my $line ( @output)
            {

                my $hash = \%$line;
                print "name: ".$hash->{name}." fullname: ".$hash->{fullname}.", manufacturer: ".$hash->{manufacturer}.", type: ".$hash->{type}.", version: ".$hash->{version}."\n";
            }
}
exit($return);
